#include "windows.h"   /* required for all Windows applications */
#ifdef __cplusplus
extern "C"
{
#endif
#include <string.h>
#include "P3Types.h"   /* Phase3 miscellaneous types     */
#include "P3Procs.h"   /* Phase3 miscellaneous functions */
#ifdef __cplusplus
}
#endif

extern HANDLE hInst;

#ifdef __cplusplus
extern "C"
{
   BOOL WINAPI SystemWarning (LPSTR szMessage);
   void LongWarning (LPSTR szTitle, LPSTR szMessage);
   void P3DrawLine (HDC hdcLine, int nPenWidth, COLORREF lPenColor,
                    int x1, int y1, int x2, int y2);
   void P3DrawObject (HDC hdcObj, int nPenWidth, COLORREF lPenColor,
                      int x1, int y1, int x2, int y2, BOOL bRectangle);
   void P3DrawFilled (HDC hdcObj, int nPenWidth,
                      COLORREF lPenColor, COLORREF lBrushColor,
                      int x1, int y1, int x2, int y2, BOOL bRectangle);
   void P3OwnerDrawButton (DRAWITEMSTRUCT FAR * P3_lpDIS,
                           char * szSuffix, WORD wStyle);
}
#endif

char *LongWarningStr[10] = { "Global lock error.",
                             "Invalid extra bytes pointer.",
                             "Cannot create window for object.",
                             "Cannot get font for object.",
                             "Bitmap not found.",
                             "Out of memory.",
                             "Cannot get a device context.",
                             "Could not create window.",
                             "Could not create dialog box.",
                             "Could not register window." };


/*=-----------------------------------------------------=*/
BOOL WINAPI SystemWarning (LPSTR szMessage)
/*=-----------------------------------------------------=*/
   {
   MessageBeep(0);
   MessageBox (0, szMessage, "System Error", MB_ICONSTOP | MB_TASKMODAL);
   return TRUE;
   }


/*=-----------------------------------------------------=*/
void LongWarning (LPSTR szTitle, LPSTR szMessage)
/*=-----------------------------------------------------=*/
   {
   MessageBeep(0);
   MessageBox (0, szMessage, szTitle, MB_ICONSTOP | MB_TASKMODAL);
   }


/*=-----------------------------------------------------=*/
void P3DrawLine (HDC hdcLine, int nPenWidth, COLORREF lPenColor,
                 int x1, int y1, int x2, int y2)
/*=-----------------------------------------------------=*/
/*
 *  Draw a line on given device context.
 */

{
   HPEN hPenWork;

   hPenWork = SelectObject (hdcLine, CreatePen (PS_SOLID, nPenWidth, lPenColor));
   MoveTo (hdcLine, x1, y1);
   LineTo (hdcLine, x2, y2);
   DeleteObject (SelectObject (hdcLine, hPenWork));
}


/*=-----------------------------------------------------=*/
void P3DrawObject (HDC hdcObj, int nPenWidth, COLORREF lPenColor,
                   int x1, int y1, int x2, int y2, BOOL bRectangle)
/*=-----------------------------------------------------=*/
/*
 *  Draw a rectangle or circle on given device context. If input
 *  parameter "bRectangle" is TRUE then draw a rectangle, otherwise
 *  draw a circle.
 */

{
   HPEN hPenWork;

   SelectObject (hdcObj, GetStockObject (NULL_BRUSH));
   hPenWork = SelectObject (hdcObj, CreatePen (PS_SOLID, nPenWidth, lPenColor));
   if (bRectangle)
      Rectangle (hdcObj, x1, y1, x2, y2);
   else
      Ellipse (hdcObj, x1, y1, x2, y2);
   DeleteObject (SelectObject (hdcObj, hPenWork));
}


/*=-----------------------------------------------------=*/
void P3DrawFilled (HDC hdcObj, int nPenWidth,
                   COLORREF lPenColor, COLORREF lBrushColor,
                   int x1, int y1, int x2, int y2, BOOL bRectangle)
/*=-----------------------------------------------------=*/
/*
 *  Draw a filled rectangle or circle on given device context. If input
 *  parameter "bRectangle" is TRUE then draw a filled rectangle, otherwise
 *  draw a filled circle.
 */

{
   HPEN   hPenWork;
   HBRUSH hBrushWork;

   hPenWork = SelectObject (hdcObj, CreatePen (PS_SOLID, nPenWidth, lPenColor));
   hBrushWork = SelectObject (hdcObj, CreateSolidBrush (lBrushColor));
   if (bRectangle)
      Rectangle (hdcObj, x1, y1, x2, y2);
   else
      Ellipse (hdcObj, x1, y1, x2, y2);
   DeleteObject (SelectObject (hdcObj, hBrushWork));
   DeleteObject (SelectObject (hdcObj, hPenWork));
}


/*=-----------------------------------------------------=*/
void P3OwnerDrawButton (DRAWITEMSTRUCT FAR * P3_lpDIS,
                        char * szSuffix,
                        WORD wStyle)
/*=-----------------------------------------------------=*/
/*
 *  Display an owner draw button in appropriate state.
 */

{
   char    szBitmap[20];
   char    szExt[3];
   HBITMAP P3_hBMP;

   if (P3_lpDIS->CtlType != ODT_BUTTON)
      return;

      /* Select bitmap to match state of owner draw button */
   strcpy (szBitmap, "ODB_");
   strcpy (szExt, "N_");
   switch (P3_lpDIS->itemAction)
   {
      case ODA_DRAWENTIRE :
         if ((P3_lpDIS->itemState & ODS_DISABLED) == ODS_DISABLED)
             strcpy (szExt, "D_");
         else
            if ((P3_lpDIS->itemState & ODS_FOCUS) == ODS_FOCUS)
                strcpy (szExt, "F_");
         break;
      case ODA_SELECT :
         if ((P3_lpDIS->itemState & ODS_SELECTED) == ODS_SELECTED)
             strcpy (szExt, "S_");
         else
            if ((P3_lpDIS->itemState & ODS_FOCUS) == ODS_FOCUS)
                strcpy (szExt, "F_");
         break;
      case ODA_FOCUS :
         if ((P3_lpDIS->itemState & ODS_FOCUS) == ODS_FOCUS)
             strcpy (szExt, "F_");
         break;
      default :
         return;
   }

      /* Draw appropriate bitmap for owner draw button */
   strcat (szBitmap, szExt);
   strcat (szBitmap, szSuffix);
   if ((P3_hBMP = LoadBitmap (hInst, szBitmap)) != 0)
      {
      P3_BitmapPaintWindow (P3_lpDIS->hwndItem, P3_hBMP, wStyle, SRCCOPY);
      DeleteObject (P3_hBMP);
      }
}
